import alphatwirl
import unittest

##__________________________________________________________________||
# How to run test
# at the directory three up (under AlphaTwirl/)
#
# to run all tests in this file
# $ python -m unittest tests.unit.examples.test_test_example
#
# to run a particular test in this file
# $ python -m unittest tests.unit.examples.test_test_example.TestExample1
#
# to run all tests in all files
# $ python -m unittest discover

##__________________________________________________________________||
class TestExample1(unittest.TestCase):
    def test_example(self):
        pass
    pass

##__________________________________________________________________||
class TestExample2(unittest.TestCase):
    def test_example(self):
        pass
    pass

##__________________________________________________________________||
