
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2019-01-25T11:47:16+0000",
 "dirty": false,
 "error": null,
 "full-revisionid": "da6a460bd0c6ec38844e7471b0fa23792df0965e",
 "version": "0.20.3"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
